//Datei: CMESS.C
// Hauptprogramm
// C-Megert mit PIC16F84
//
// Holger Klabunde
// Copyright 2000
// 01.05.2001
// Compiler PICC Lite

// Achtung:	int ist 16 Bit lang
//		char ist 8 Bit lang
//		long ist 32 Bit lang

// Der Takt des PIC betrgt 10MHz
// Ein Maschinenzyklus damit 400ns

// Verwendete LCD-Anzeige 1x16 Zeichen
// Bentigt werden nur 1x8 Zeichen !!

#include "prozess.h"      /* Der verwendete Prozessor */

#include "cmess.h"
#include "protos.h"

 // Watchdog off, Power up Timer on, HS Oscillator, Code Protect off
 __CONFIG(0x3FF2);

//Globale Flags und Rckgabevariablen

unsigned int count; //16Bit Zhler
unsigned char count1; //Puffer fr TMR0
unsigned char offs;   //Fr Messung Eingangskapazitt
unsigned int offs1;  //Fr Messung Eingangskapazitt

void MeasureC();
void CinKompensation();


/* Hauptprogramm */
void main()
{
 InitPIC(); //Interrupts und so weiter
 LCDInit(); //LCD Anzeige einstellen

 LCDCls();  //Anzeige lschen
 LCDWrite("Holgis");
 LCDPos(2,1);
 LCDWrite("C Messer");
 Delay1ms(255);          //Anzeige etwas verlngern
 Delay1ms(255);          //Anzeige etwas verlngern
 Delay1ms(255);          //Anzeige etwas verlngern
 Delay1ms(255);          //Anzeige etwas verlngern
 LCDCls();  //Anzeige lschen

//Kapazitt bei offenem Eingang messen
 MeasureC();
 offs=count1; // 8 Bit LSB
 offs1=count; //16 Bit MSW
//Ende Kapazitt bei offenem Eingang messen
  
 while(1) // Anfang Programm Endlosschleife
  {
   if(NULLAB==0) //Taste Nullabgleich gedrckt
    {
     LCDCls();  //Anzeige lschen
     LCDWrite("Abgleich");
     while(NULLAB==0); //warten bis Taste losgelassen wird
     Delay1ms(255);          //Anzeige etwas verlngern
     Delay1ms(255);          //Anzeige etwas verlngern
     //Kapazitt bei offenem Eingang messen
     MeasureC();
     offs=count1; // 8 Bit LSB
     offs1=count; //16 Bit MSW
     LCDCls();  //Anzeige lschen
    }

   MeasureC(); //Kapazitt messen
   CinKompensation(); //Cin abziehen

   if(HOLD==0)
    {
     LCDPos(2,1);
     ShowDisplay();
     while(HOLD==0); //warten bis Taste losgelassen wird
    }
   
   //Zhler anzeigen
  LCDPos(1,1);
  ShowDisplay();

  } // Ende Programm Endlosschleife
}

//Kapazitt messen
void MeasureC()
{
   count=0; //Zhler zurcksetzen
   RST=0;   //Messmonoflop zurcksetzen
   Delay1ms(50); //Warten bis C entladen ist ca. 15ms bei RB=1k
//   while(CCLK==0); //Warten auf negative Flanke
   RST=1;   //Messmonoflop starten
   
//Eine ganze Periode messen falls Tastverhltnis unsymmetrisch
   TMR0=0;
   T0IE=1; //Interrupt erlauben
   while(CCLK==1); //Lnge der -Periode messen
//   while(CCLK==1); //Lnge der +Periode messen
//   while(CCLK==0); //Lnge der -Periode messen
   count1=TMR0;       //Wert schnellstens sichern
   T0IE=0; //Interrupt verbieten
                  //Sonst zhlt count weiter !!
//Ein zweites Mal falls oben Int auftauchte
   T0IE=0; //Interrupt verbieten
                  //Sonst zhlt count weiter !!
}


//************************************************
// Zieht die Eingangskapazitt vom Ergebnis ab 
//************************************************
void CinKompensation()
{
//Eingangskapazitt abziehen
 if(offs<=count1) count1-=offs;
 else //offs ist grer als count1
  {   //dann bertrag in count bercksichtigen
   count-=1; //bertrag abziehen
   count1-=offs; //offs abziehen
  }

 count-=offs1; //zweiten Teil der Eingangskapazitt abziehen
}

//************************************************
// Zeigt die 5 Dezimalstellen auf dem LCD Display 
//************************************************
void ShowDisplay()
{
 unsigned char mode; //feststellen welche Einheit angezeigt wird
 unsigned char b;
 
 mode=0; //Annehmen Anzeige in pF
 
// LCDPos(1,1);

//Integer zu BCD und anzeigen
 B0=count1; //Lowbyte
 B1=(unsigned char)count;      //Midbyte
 B2=(unsigned char)(count>>8); //Highbyte
 BinToBCD24(); //Binr wandeln in BCD Bytes

 b=(R2>>4)&0x0F;
 if(b>0) mode=1; //nF Stellen grer als 0
 if(R1>0) mode=1; //nF Stellen grer als 0
 if(R0>0) mode=2; //uF Stellen grer als 0

 if(mode==2) //uF
  {
   if(R0>0x0F) //Wert ber 10uF zwei uF Stellen
    {
     ShowBCD(R0);
     LCDWrite(".");
     ShowBCD(R1);
     b=(R2>>4)&0x0F; //Einerstelle nF
     b+=0x30; //In Ascii wandeln
     LCDWriteByte(b);
    }
   else  //Wert unter 10uF eine uF Stelle
    {
     b=R0&0x0F; //1er Stelle uF
     b+=0x30; //In Ascii wandeln
     LCDWriteByte(b);
     LCDWrite(".");
     ShowBCD(R1);
     ShowBCD(R2);
    }
   LCDWrite("uF");
  }
   
 if(mode==1)
  {
   if(R1>0x0F) //Wert ber 100nF drei nF Stellen
    {
     ShowBCD(R1);
     b=(R2>>4)&0x0F; //Einerstelle nF
     b+=0x30; //In Ascii wandeln
     LCDWriteByte(b);
     LCDWrite(".");
     b=R2&0x0F; //100ter Stelle pF
     b+=0x30; //In Ascii wandeln
     LCDWriteByte(b);
     b=(R3>>4)&0x0F; //10er Stelle pF
     b+=0x30; //In Ascii wandeln
     LCDWriteByte(b);
    }
   else
    {
     if((R1&0x0F)>0)  //Wert unter 100nF zwei nF Stellen
      {
       b=R1&0x0F; //10er Stelle nF
       b+=0x30; //In Ascii wandeln
       LCDWriteByte(b);

       b=(R2>>4)&0x0F; //Einerstelle nF
       b+=0x30; //In Ascii wandeln
       LCDWriteByte(b);
       LCDWrite(".");
       b=R2&0x0F; //100ter Stelle pF
       b+=0x30; //In Ascii wandeln
       LCDWriteByte(b);
       ShowBCD(R3);
      }
     else //Wert unter 10nF eine nF Stelle
      {
       LCDWriteByte(0x20); //Ein Space
       b=(R2>>4)&0x0F; //Einerstelle nF
       b+=0x30; //In Ascii wandeln
       LCDWriteByte(b);
       LCDWrite(".");
       b=R2&0x0F; //100ter Stelle pF
       b+=0x30; //In Ascii wandeln
       LCDWriteByte(b);
       ShowBCD(R3);
      } 
    }

   LCDWrite("nF");
  }

 if(mode==0)
  {
   LCDWrite("   "); //Damit Einheit immer an derselben Stelle ist
   b=R2&0x0F; //100ter Stelle pF
   if(b>0) b+=0x30; //In Ascii wandeln
   else b=0x20;    //Sonst Leerzeichen
   LCDWriteByte(b);
   ShowBCD(R3); 
   LCDWrite("pF");
  }
  
 Delay1ms(200);  //Damit die Anzeige nicht flackert
}
